@@ -130,13 +130,13 @@ mcmc_parcoord <-
     div_draws <- dplyr::filter(data, UQ(divg) == 1)
     has_divs <- isTRUE(nrow(div_draws) > 0)
 
-    graph <- ggplot(draws, aes_(
-      x = ~ Parameter,
-      y = ~ Value,
-      group = ~ factor(Draw)
+    graph <- ggplot(draws, aes(
+      x = .data$Parameter,
+      y = .data$Value,
+      group = factor(.data$Draw)
     )) +
       geom_line(
-        size = size,
+        linewidth = size,
         alpha = alpha,
         color = get_color("dh")
       ) +
@@ -146,7 +146,7 @@ mcmc_parcoord <-
       graph <- graph +
         geom_line(
           data = div_draws,
-          size = np_style$size[["div"]],
+          linewidth = np_style$size[["div"]],
           alpha = np_style$alpha[["div"]],
           color = np_style$color[["div"]]
         )
@@ -154,6 +154,7 @@ mcmc_parcoord <-
 
     graph +
       scale_x_discrete(expand = c(0,0), labels = levels(draws$Parameter)) +
+      expand_limits(x = nlevels(draws$Parameter) + 0.25) +
       labs(x = NULL, y = NULL)
   }
 
