@@ -9,7 +9,12 @@
 #' @template args-y-yrep
 #' @template args-group
 #' @template args-facet_args
+#' @param x A numeric vector the same length as `y` to use as the x-axis variable.
 #' @param ... Currently unused.
+#' @param stat A function or a string naming a function for computing the
+#' posterior average. In both cases, the function should take a vector input and
+#' return a scalar statistic. The function name is displayed in the axis-label.
+#' Defaults to `"mean"`.
 #' @param size,alpha For scatterplots, arguments passed to
 #'   [ggplot2::geom_point()] to control the appearance of the points. For the
 #'   binned error plot, arguments controlling the size of the outline and
@@ -49,10 +54,12 @@
 #'    `y` and each dataset (row) in `yrep`. For each individual data point
 #'    `y[n]` the average error is the average of the errors for `y[n]` computed
 #'    over the the draws from the posterior predictive distribution.
+#'
+#'    When the optional `x` argument is provided, the average error is plotted
+#'    on the y-axis and the predictor variable `x` is plotted on the x-axis.
 #'   }
 #'   \item{`ppc_error_scatter_avg_vs_x()`}{
-#'    Same as `ppc_error_scatter_avg()`, except the average is plotted on the
-#'    y-axis and a predictor variable `x` is plotted on the x-axis.
+#'    Deprecated. Use `ppc_error_scatter_avg(x = x)` instead.
 #'   }
 #'   \item{`ppc_error_binned()`}{
 #'    Intended for use with binomial data. A separate binned error plot (similar
@@ -87,7 +94,7 @@
 #' ppc_error_scatter_avg(y, yrep)
 #'
 #' x <- example_x_data()
-#' ppc_error_scatter_avg_vs_x(y, yrep, x)
+#' ppc_error_scatter_avg(y, yrep, x)
 #'
 #' \dontrun{
 #' # binned error plot with binomial model from rstanarm
@@ -105,6 +112,10 @@
 #' yrep_prop <- sweep(yrep, 2, trials, "/")
 #'
 #' ppc_error_binned(y_prop, yrep_prop[1:6, ])
+#'
+#' # plotting against a covariate on x-axis
+#' herd <- as.numeric(example_model$data$herd)
+#' ppc_error_binned(y_prop, yrep_prop[1:6, ], x = herd)
 #' }
 #'
 NULL
@@ -134,7 +145,7 @@ ppc_error_hist <-
       geom_histogram(
         fill = get_color("l"),
         color = get_color("lh"),
-        size = 0.25,
+        linewidth = 0.25,
         binwidth = binwidth,
         bins = bins,
         breaks = breaks
@@ -208,22 +219,40 @@ ppc_error_scatter <-
 ppc_error_scatter_avg <-
   function(y,
            yrep,
+           x = NULL,
            ...,
+           stat = "mean",
            size = 2.5,
            alpha = 0.8) {
     check_ignored_arguments(...)
 
     y <- validate_y(y)
     yrep <- validate_predictions(yrep, length(y))
+
+    if (!missing(x)) {
+      qx <- enquo(x)
+      x <- validate_x(x, y)
+    }
     errors <- compute_errors(y, yrep)
+
+    stat <- as_tagged_function({{ stat }})
+
     ppc_scatter_avg(
-      y = y,
+      y = if (is_null(x)) y else x,
       yrep = errors,
       size = size,
       alpha = alpha,
-      ref_line = FALSE
+      ref_line = FALSE,
+      stat = stat
     ) +
-      labs(x = error_avg_label(), y = y_label())
+      labs(
+        x = error_avg_label(stat),
+        y = if (is_null(x)) y_label() else as_label((qx))
+        ) + if (is_null(x)) {
+          NULL
+        } else {
+          coord_flip()
+        }
   }
 
 
@@ -234,6 +263,7 @@ ppc_error_scatter_avg_grouped <-
            yrep,
            group,
            ...,
+           stat = "mean",
            facet_args = list(),
            size = 2.5,
            alpha = 0.8) {
@@ -241,6 +271,8 @@ ppc_error_scatter_avg_grouped <-
 
     y <- validate_y(y)
     yrep <- validate_predictions(yrep, length(y))
+    stat <- as_tagged_function({{ stat }})
+
     errors <- compute_errors(y, yrep)
     ppc_scatter_avg_grouped(
       y = y,
@@ -249,40 +281,48 @@ ppc_error_scatter_avg_grouped <-
       size = size,
       alpha = alpha,
       facet_args = facet_args,
-      ref_line = FALSE
+      ref_line = FALSE,
+      stat = stat
     ) +
-      labs(x = error_avg_label(), y = y_label())
+      labs(x = error_avg_label(stat), y = y_label())
   }
 
 
 #' @rdname PPC-errors
 #' @export
-#' @param x A numeric vector the same length as `y` to use as the x-axis
-#'   variable.
-#'
-ppc_error_scatter_avg_vs_x <-
-  function(y,
-           yrep,
-           x,
-           ...,
-           size = 2.5,
-           alpha = 0.8) {
-    check_ignored_arguments(...)
+ppc_error_scatter_avg_vs_x <- function(
+    y,
+    yrep,
+    x,
+    ...,
+    stat = "mean",
+    size = 2.5,
+    alpha = 0.8
+) {
+  check_ignored_arguments(...)
 
-    y <- validate_y(y)
-    yrep <- validate_predictions(yrep, length(y))
-    x <- validate_x(x, y)
-    errors <- compute_errors(y, yrep)
-    ppc_scatter_avg(
-      y = x,
-      yrep = errors,
-      size = size,
-      alpha = alpha,
-      ref_line = FALSE
+  .Deprecated(new = "ppc_error_scatter_avg(y, yrep, x)")
+
+  y <- validate_y(y)
+  yrep <- validate_predictions(yrep, length(y))
+  qx <- enquo(x)
+  x <- validate_x(x, y)
+  stat <- as_tagged_function({{ stat }})
+  errors <- compute_errors(y, yrep)
+  ppc_scatter_avg(
+    y = x,
+    yrep = errors,
+    size = size,
+    alpha = alpha,
+    ref_line = FALSE,
+    stat = stat
+  ) +
+    labs(
+      x = error_avg_label(stat),
+      y = as_label((qx))
     ) +
-      labs(x = error_avg_label(), y = expression(italic(x))) +
-      coord_flip()
-  }
+    coord_flip()
+}
 
 
 #' @rdname PPC-errors
@@ -291,6 +331,7 @@ ppc_error_scatter_avg_vs_x <-
 ppc_error_binned <-
   function(y,
            yrep,
+           x = NULL,
            ...,
            facet_args = list(),
            bins = NULL,
@@ -298,7 +339,8 @@ ppc_error_binned <-
            alpha = 0.25) {
     check_ignored_arguments(...)
 
-    data <- ppc_error_binnned_data(y, yrep, bins = bins)
+    qx <- enquo(x)
+    data <- ppc_error_binnned_data(y, yrep, x = x, bins = bins)
     facet_layer <- if (nrow(yrep) == 1) {
       geom_ignore()
     } else {
@@ -335,7 +377,7 @@ ppc_error_binned <-
         color = point_color
       ) +
       labs(
-        x = "Predicted proportion",
+        x = if (is.null(x)) "Predicted proportion" else as_label((qx)),
         y = "Average Errors \n (with 2SE bounds)"
       ) +
       bayesplot_theme_get() +
@@ -414,16 +456,33 @@ error_hist_facets <-
 error_label <- function() {
   expression(italic(y) - italic(y)[rep])
 }
-error_avg_label <- function() {
-  expression(paste("Average ", italic(y) - italic(y)[rep]))
+
+error_avg_label <- function(stat = NULL) {
+  stat <- as_tagged_function({{ stat }}, fallback = "stat")
+  e <- attr(stat, "tagged_expr")
+  if (attr(stat, "is_anonymous_function")) {
+    e <- sym("stat")
+  }
+  de <- deparse1(e)
+
+  # create some dummy variables to pass the R package check for
+  # global variables in the expression below
+  italic <- sym("italic")
+  y <- sym("y")
+
+  expr(paste((!!de))*(italic(y) - italic(y)[rep]))
 }
 
 
 # Data for binned errors plots
-ppc_error_binnned_data <- function(y, yrep, bins = NULL) {
+ppc_error_binnned_data <- function(y, yrep, x = NULL, bins = NULL) {
   y <- validate_y(y)
   yrep <- validate_predictions(yrep, length(y))
 
+  if (!is.null(x)) {
+    x <- validate_x(x, y)
+  }
+
   if (is.null(bins)) {
     bins <- n_bins(length(y))
   }
@@ -431,13 +490,24 @@ ppc_error_binnned_data <- function(y, yrep, bins = NULL) {
   errors <- compute_errors(y, yrep)
   binned_errs <- list()
   for (s in 1:nrow(errors)) {
-    binned_errs[[s]] <-
-      bin_errors(
-        ey = yrep[s, ],
-        r = errors[s, ],
-        bins = bins,
-        rep_id = s
-      )
+    if (is.null(x)) {
+      binned_errs[[s]] <-
+        bin_errors(
+          ey = yrep[s, ],
+          r = errors[s, ],
+          bins = bins,
+          rep_id = s
+        )
+    } else {
+      binned_errs[[s]] <-
+        bin_errors(
+          ey = x,
+          r = errors[s, ],
+          bins = bins,
+          rep_id = s
+        )
+    }
+
   }
 
   binned_errs <- dplyr::bind_rows(binned_errs)
