@@ -1,10 +1,5 @@
-library(bayesplot)
-context("MCMC: misc. functions")
-
 source(test_path("data-for-mcmc-tests.R"))
 
-
-
 # melt_mcmc ----------------------------------------------------------------
 test_that("melt_mcmc does not convert integer parameter names to integers #162", {
   mat2 <- mat[, 1:2]
@@ -303,7 +298,11 @@ test_that("diagnostic_factor.rhat works", {
                       high = 1.2, high = 1.7))
 
   r <- diagnostic_factor(unname(rhats))
-  expect_equivalent(r, as.factor(names(rhats)))
+  expect_equal(
+    r,
+    factor(names(rhats), levels = c("low", "ok", "high")),
+    ignore_attr = TRUE
+  )
   expect_identical(levels(r), c("low", "ok", "high"))
 })
 test_that("diagnostic_factor.neff_ratio works", {
@@ -312,7 +311,11 @@ test_that("diagnostic_factor.neff_ratio works", {
                              high = 0.51, high = 0.99, high = 1))
 
   r <- diagnostic_factor(unname(ratios))
-  expect_equivalent(r, as.factor(names(ratios)))
+  expect_equal(
+    r,
+    factor(names(ratios), levels = c("low", "ok", "high")),
+    ignore_attr = TRUE
+  )
   expect_identical(levels(r), c("low", "ok", "high"))
 })
 
