@@ -1,6 +1,3 @@
-library(bayesplot)
-context("PPC: input validation")
-
 source(test_path("data-for-ppc-tests.R"))
 
 test_that("validate_y works", {
@@ -23,7 +20,7 @@ test_that("validate_predictions works", {
   expect_equal(validate_predictions(yrep2, length(y2)), yrep2)
 
   colnames(yrep) <- paste0("yrep", 1:ncol(yrep))
-  expect_equivalent(validate_predictions(yrep, length(y)), unname(yrep))
+  expect_equal(validate_predictions(yrep, length(y)), unname(yrep), ignore_attr = TRUE)
 })
 test_that("validate_predictions throws errors", {
   expect_error(validate_predictions(as.matrix(LETTERS), length(y)), "numeric")
